/*
/*
 *  Copyright (c) by CryptoSoft GmbH 1998-2017
 *  All Rights Reserved
 *  Licensed Material - Property of CryptoSoft GmbH
 *  This software is made available solely pursuant to the
 *  terms of a license agreement which governs its use.
 *
 *  PTHREAD.C - POSIX Thread Test
 */

#if defined(WIN16) || defined(WIN32) || defined(WIN64)
#include <windows.h>
#endif

#include <stdio.h>       /* standard I/O routines                 */
#include <string.h>      /* standard mem routines                 */
#include <pthread.h>     /* pthread functions and data structures */
#include "safer.h"       /* Safer function prototypes */

/* size of key, in bytes */
#define KEYSZ	(32)
/* size of standard block, in bytes */
#define	BLKSZ	(16)

static char *algo  = "Safer+";
static int verbose = 0;

/* Variable Key Known Answer Thread */
void* do_vk(void* data)
{
  SAFER_KS ks;
  unsigned char kb[KEYSZ], pt[BLKSZ], ct[BLKSZ], ot[BLKSZ];
  unsigned int i, j, k;

  int me = *((int*)data);                   /* thread identifying number */

  memset(pt, 0, BLKSZ);

  for (i = 0; i < 10; i++) {
    for (j = 0; j < KEYSZ * 8; j++) {
      for (k = 0; k < 5000; k++) ;         /* delay */
      memset(kb, 0, KEYSZ);
      kb[j/8] = 0x80 >> j%8;
      safer_init(kb, KEYSZ, &ks);
      safer_ecbencode(pt, ct, &ks);
      safer_ecbdecode(ct, ot, &ks);
      if (memcmp(pt, ot, BLKSZ)) {
        printf("\nVT Thread '%d' - Got error in loop '%d'", me, i);
        printf("\nKEY="); for (k = 0; k < KEYSZ; k++) printf("%02x", kb[k]);
        printf("\nPT=");  for (k = 0; k < BLKSZ; k++) printf("%02x", pt[k]);
        printf("\nCT=");  for (k = 0; k < BLKSZ; k++) printf("%02x", ct[k]);
        printf("\nOT=");  for (k = 0; k < BLKSZ; k++) printf("%02x", ot[k]);
        pthread_exit(NULL);
      }
    }
    if (verbose) printf("\nVK Thread '%d' - loop '%d' okay", me, i);
  }
  /* terminate the thread */
  pthread_exit(NULL); 
}

/* Variable Text Known Answer Thread */
void* do_vt(void* data)
{
  SAFER_KS ks;
  unsigned char kb[KEYSZ], pt[BLKSZ], ct[BLKSZ], ot[BLKSZ];
  unsigned int i, j, k;

  int me = *((int*)data);                   /* thread identifying number */

  memset(kb, 0, KEYSZ);

  for (i = 0; i < 10; i++) {
    for (j = 0; j < BLKSZ * 8; j++) {
      for (k = 0; k < 5000; k++) ;         /* delay */
      memset(pt, 0, BLKSZ);
      pt[i/8] = 0x80 >> i%8;
      safer_init(kb, KEYSZ, &ks);
      safer_ecbencode(pt, ct, &ks);
      safer_ecbdecode(ct, ot, &ks);
      if (memcmp(pt, ot, BLKSZ)) {
        printf("\nVT Thread '%d' - Got error in loop '%d'", me, i);
        printf("\nKEY="); for (k = 0; k < KEYSZ; k++) printf("%02x", kb[k]);
        printf("\nPT=");  for (k = 0; k < BLKSZ; k++) printf("%02x", pt[k]);
        printf("\nCT=");  for (k = 0; k < BLKSZ; k++) printf("%02x", ct[k]);
        printf("\nOT=");  for (k = 0; k < BLKSZ; k++) printf("%02x", ot[k]);
        pthread_exit(NULL);
      }
    }
    if (verbose) printf("\nVT Thread '%d' - loop '%d' okay", me, i);
  }
  /* terminate the thread */
  pthread_exit(NULL);
}


int main(int argc, char **argv)
{
  pthread_t  p_thread;       /* thread's structure                     */
  int        a         = 1;  /* thread 1 identifying number            */
  int        b         = 2;  /* thread 2 identifying number            */
  int        c         = 3;  /* thread 3 identifying number            */


  /* do we want some output ? */
  if (argc > 1) if (strcmp(*++argv, "-v") == 0) verbose = 1;

  if (verbose) printf("Algorithm: %s\nKeysize=%u Bits", algo, KEYSZ*8);

  /* create a first thread that will execute 'do_vt()' */
  pthread_create(&p_thread, NULL, do_vt, (void*)&a);
  if (verbose) printf("\nVT Thread 1 created and started");
  /* create a second thread that will execute 'do_vk()' */
  pthread_create(&p_thread, NULL, do_vk, (void*)&a);
  if (verbose) printf("\nVK Thread 1 created and started");
  /* create a third thread that will execute 'do_vt()' */
  pthread_create(&p_thread, NULL, do_vt, (void*)&b);
  if (verbose) printf("\nVT Thread 2 created and started");
  /* create a fourth thread that will execute 'do_vk()' */
  pthread_create(&p_thread, NULL, do_vk, (void*)&b);
  if (verbose) printf("\nVK Thread 2 created and started");
  /* run both threads in the main thread as well */
  do_vt((void*)&c);
  /* NOT REACHED */
  do_vk((void*)&c); 
  return 0;
}
